(function() {
  var clean, commandWraps, compile, destroy, dumps_folder_name, exec, existsSync, extname, filterCoffee, filterDirectory, generate, generateFile, getConfig, getDirectories, getFiles, getTimestamp, getVersion, gets, gitIgnore, grindCoffee, handleCommand, handleFile, handleFilter, handleList, handleOutput, handleShow, handleView, help, join, listCoffee, log, mapCoffee, missingPythonDeps, mkdirSync, operateOn, padTwo, prepare, print, processDirectory, processOptions, readFileSync, readdirSync, reduceCoffee, releases_folder_name, restore, showCoffee, showGreatings, spawn, statSync, symlinkSync, unlinkSync, writeFileSync, _, _ref, _ref2, _ref3, _ref4;
  var __slice = Array.prototype.slice;
  _ref = require('path'), existsSync = _ref.existsSync, join = _ref.join, extname = _ref.extname;
  _ref2 = require('fs'), mkdirSync = _ref2.mkdirSync, readdirSync = _ref2.readdirSync, writeFileSync = _ref2.writeFileSync, readFileSync = _ref2.readFileSync, statSync = _ref2.statSync, symlinkSync = _ref2.symlinkSync, unlinkSync = _ref2.unlinkSync;
  compile = require('coffee-script').compile;
  _ref3 = require('child_process'), exec = _ref3.exec, spawn = _ref3.spawn;
  _ref4 = require('util'), print = _ref4.print, gets = _ref4.gets;
  log = console.log;
  _ = require('underscore')._;
  dumps_folder_name = '.dumps';
  releases_folder_name = '.releases';
  commandWraps = [
    {
      name: 'push',
      type: 'before',
      callback: function() {
        return grindCoffee();
      }
    }, {
      name: 'help',
      type: 'after',
      desc: '     show this message',
      callback: function() {
        return help();
      }
    }, {
      name: 'cgenerate',
      type: 'before',
      desc: '[ view | list | show | filter ] generate .coffee versions',
      callback: function() {
        return generate();
      }
    }, {
      name: 'destroy',
      type: 'before',
      desc: '  [ view | list | show | filter ] destroy (remove directory/files also .js files).',
      callback: function() {
        return destroy();
      }
    }, {
      name: 'prepare',
      type: 'before',
      desc: '  prepare (.gitignore...)',
      callback: function() {
        return prepare();
      }
    }, {
      name: 'clean',
      type: 'before',
      desc: "    remove " + releases_folder_name + " & " + dumps_folder_name + " directories",
      callback: function() {
        return clean();
      }
    }, {
      name: 'restore',
      type: 'before',
      desc: '  restore database from #{dumps_folder_name}/last',
      callback: function() {
        return restore();
      }
    }
  ];
  gitIgnore = ".DS_Store\n.couchapprc\n" + dumps_folder_name + "/*\n" + dumps_folder_name + "/**/*\n" + releases_folder_name + "/*\n" + releases_folder_name + "/**/*";
  mapCoffee = '(doc) ->\n  ...';
  reduceCoffee = '(keys, values, rereduce) ->\n  ...';
  listCoffee = '(head, req) ->\n  ...';
  showCoffee = '(doc, req) ->\n  ...';
  filterCoffee = '(doc, req) ->\n  ...';
  getVersion = function() {
    return readFileSync(join(__dirname, '..', 'package.json')).toString().match(/"version"\s*:\s*"([\d.]+)"/)[1];
  };
  showGreatings = function() {
    log("CoffeeApp (" + (getVersion()) + ") - simple coffee-script wrapper for CouchApp (http://couchapp.org)");
    return log('http://github.com/andrzejsliwa/coffeeapp\n');
  };
  padTwo = function(number) {
    var result;
    result = number < 10 ? '0' : '';
    return "" + result + number;
  };
  getTimestamp = function() {
    var date;
    date = new Date;
    return date.getFullYear() + padTwo(date.getMonth() + 1) + padTwo(date.getDate()) + padTwo(date.getHours() + 1) + padTwo(date.getMinutes() + 1) + padTwo(date.getSeconds() + 1);
  };
  handleOutput = function(callbackOk, callbackError) {
    return function(error, stdout, stderr) {
      if (error !== null) {
        if (callbackError !== null) {
          return callbackError();
        } else {
          if (stderr && stderr.length > 0) {
            log(stderr);
          }
          return log("exec error: " + error);
        }
      } else {
        if (callbackOk !== null) {
          return callbackOk();
        } else {
          if (stdout && stdout.length > 0) {
            return log(stdout);
          }
        }
      }
    };
  };
  getConfig = function() {
    return JSON.parse(readFileSync('.couchapprc', 'utf8'));
  };
  getDirectories = function(currentDir, ignores) {
    var callback;
    callback = function(name) {
      return statSync(join(currentDir, name)).isDirectory();
    };
    return filterDirectory(currentDir, callback, ignores);
  };
  getFiles = function(currentDir, ignores) {
    var callback;
    callback = function(name) {
      return !statSync(join(currentDir, name)).isDirectory();
    };
    return filterDirectory(currentDir, callback, ignores);
  };
  filterDirectory = function(currentDir, callback, ignores) {
    var list, results;
    list = readdirSync(currentDir);
    if (ignores) {
      list = _.without.apply(_, [list].concat(__slice.call(ignores)));
    }
    return results = _.filter(list, callback);
  };
  processDirectory = function(baseDir, destination) {
    var currentDir, dirs, files, isError, subDirs;
    dirs = [baseDir];
    isError = false;
    while (dirs.length > 0) {
      currentDir = dirs.pop();
      subDirs = getDirectories(currentDir, ['.git', releases_folder_name, dumps_folder_name]);
      _.each(subDirs, function(dirName) {
        var destDirPath, dirPath;
        dirPath = join(currentDir, dirName);
        dirs.push(dirPath);
        destDirPath = join(destination, join(dirPath));
        return mkdirSync(destDirPath, 0700);
      });
      files = getFiles(currentDir);
      _.each(files, function(fileName) {
        var destFilePath, filePath;
        filePath = join(currentDir, fileName);
        destFilePath = join(destination, filePath);
        if (extname(filePath) === ".coffee") {
          log(" * processing " + filePath + "...");
          try {
            return writeFileSync(destFilePath.replace(/\.coffee$/, '.js'), compile(readFileSync(filePath, 'utf8'), {
              bare: true
            }), 'utf8');
          } catch (error) {
            log("Compilation Error: " + error.message + "\n");
            return isError = true;
          }
        } else {
          return writeFileSync(destFilePath, readFileSync(filePath, 'binary'), 'binary');
        }
      });
    }
    return !isError;
  };
  grindCoffee = function() {
    var config, database, dumpsDir, dumpsPath, options, processCallback, releasePath, releasesDir, timestamp, url, _ref;
    log("Wrapping 'push' of couchapp");
    timestamp = getTimestamp();
    releasesDir = releases_folder_name;
    if (!existsSync(releasesDir)) {
      log("initialize " + releasesDir + " directory");
      mkdirSync(releasesDir, 0700);
    }
    releasePath = join(releasesDir, timestamp);
    dumpsDir = dumps_folder_name;
    if (!existsSync(dumpsDir)) {
      log("initialize " + dumpsDir + " directory");
      mkdirSync(dumpsDir, 0700);
    }
    dumpsPath = join(dumpsDir, timestamp);
    _ref = processOptions(), options = _ref[0], database = _ref[1];
    log("database : '" + database + "'");
    processCallback = function() {
      log("preparing release: " + releasePath);
      mkdirSync(releasePath, 0700);
      if (processDirectory('.', releasePath)) {
        process.chdir(releasePath);
        return exec("couchapp push " + options + " " + database, handleOutput(process.cwd));
      }
    };
    config = getConfig();
    if (config['env'][database]['make_dumps']) {
      log(" * making dump: " + dumpsPath);
      url = config['env'][database]['db'];
      return exec("couchdb-dump " + url + " > " + dumpsPath, handleOutput(function() {
        var lastPath;
        lastPath = join(dumpsDir, 'last');
        if (existsSync(lastPath)) {
          unlinkSync(lastPath);
        }
        log(" * linking dump: " + dumpsPath + " -> " + lastPath);
        symlinkSync(timestamp, "" + lastPath);
        return processCallback();
      }));
    }
  };
  processOptions = function() {
    var database, options, _i, _ref;
    _ref = process.argv.slice(1), options = 2 <= _ref.length ? __slice.call(_ref, 0, _i = _ref.length - 1) : (_i = 0, []), database = _ref[_i++];
    options = (options || []).join(' ');
    if (database !== null) {
      database = "default";
    }
    return [options, database];
  };
  help = function() {
    log("Wrapping 'help' of couchapp\n");
    showGreatings();
    return _.each(commandWraps, function(command) {
      if (command.desc) {
        return log("" + command.name + "        " + command.desc);
      }
    });
  };
  generateFile = function(path, template) {
    log(" * creating " + path + "...");
    if (existsSync(path)) {
      return log("File " + path + " already exist!");
    } else {
      return writeFileSync(path, template, 'utf8');
    }
  };
  generate = function() {
    return operateOn('generate');
  };
  destroy = function() {
    return operateOn('destroy');
  };
  operateOn = function(command) {
    var fun, generator, name;
    generator = process.argv[1];
    if (!generator) {
      log("missing name of " + command + " - [ view | list | show | filter ]");
      return;
    }
    name = process.argv[2];
    if (!name) {
      log('missing name of element');
      return;
    }
    print("Running " + generator + " " + command + ": ");
    fun = function() {
      switch (generator) {
        case 'view':
          return handleView;
        case 'show':
          return handleShow;
        case 'list':
          return handleList;
        case 'filter':
          return handleFilter;
        default:
          return function(method, name) {
            return log("unknown " + command);
          };
      }
    }();
    if (fun(command, name)) {
      return log('done.');
    }
  };
  handleView = function(method, name) {
    var mapFilePath, reduceFilePath, viewDirPath, _ref;
    if (!existsSync('views')) {
      mkdirSync('views', 0700);
    }
    viewDirPath = "views/" + name;
    _ref = [join(viewDirPath, "map.coffee"), join(viewDirPath, "reduce.coffee")], mapFilePath = _ref[0], reduceFilePath = _ref[1];
    switch (method) {
      case 'generate':
        if (existsSync(viewDirPath)) {
          log("directory '" + viewDirPath + "' already exist!");
          return false;
        } else {
          mkdirSync(viewDirPath, 0700);
          generateFile(mapFilePath, mapCoffee);
          generateFile(reduceFilePath, reduceCoffee);
          return true;
        }
        break;
      case 'destroy':
        if (existsSync(viewDirPath)) {
          log("'" + viewDirPath + "'.");
          exec("rm -r " + viewDirPath, handleOutput());
          return true;
        } else {
          log("there is no view '" + name + "' ('" + viewDirPath + "') !!!");
          return false;
        }
        break;
      default:
        throw 'unknown method';
    }
  };
  handleFile = function(method, folder, template, name) {
    var filePathCoffee, filePathJS;
    filePathCoffee = join(folder, "" + name + ".coffee");
    filePathJS = join(folder, "" + name + ".js");
    switch (method) {
      case 'generate':
        if (!existsSync(folder)) {
          mkdirSync(folder, 0700);
        }
        generateFile(filePathCoffee, template);
        return true;
        break;
      case 'destroy':
        if (existsSync(filePathCoffee)) {
          log("'" + filePathCoffee + "'.");
          exec("rm " + filePathCoffee, handleOutput());
          return true;
        } else if (existsSync(filePathJS)) {
          log("'" + filePathJS + "'.");
          exec("rm " + filePathJS, handleOutput());
          return true;
        } else {
          log("there is no '" + name + "' ('" + filePathJS + "' or '" + filePathCoffee + "') !!!");
          return false;
        }
        break;
      default:
        throw 'unknown method';
    }
  };
  handleShow = function(method, name) {
    return handleFile(method, 'shows', showCoffee, name);
  };
  handleList = function(method, name) {
    return handleFile(method, 'lists', listCoffee, name);
  };
  handleFilter = function(method, name) {
    return handleFile(method, 'filters', filterCoffee, name);
  };
  clean = function() {
    log("cleaning up:");
    log(" * remove '" + releases_folder_name + "' ...");
    return exec("rm -r " + releases_folder_name, handleOutput(function() {
      log(" * remove '" + dumps_folder_name + "' ...");
      return exec("rm -r " + dumps_folder_name, handleOutput(function() {
        return log("done.");
      }, function() {
        return log("there is no '" + dumps_folder_name + "' directory!");
      }));
    }, function() {
      return log("there is no '" + releases_folder_name + "' directory!");
    }));
  };
  prepare = function() {
    log("preparing project:");
    generateFile('.gitignore', gitIgnore);
    return log("done.");
  };
  restore = function() {
    var config, database, lastPath, options, url, _ref;
    _ref = processOptions(), options = _ref[0], database = _ref[1];
    config = getConfig();
    lastPath = join(dumps_folder_name, 'last');
    if (config['env'][database]['make_dumps']) {
      log(" * restoring dump from " + lastPath + " to database: ");
      url = config['env'][database]['db'];
      return exec("couchdb-load " + url + " --input " + lastPath, handleOutput(function() {
        return log("done.");
      }));
    } else {
      return log("you don't using dumps for this database... look in couchapprc for make_dumps.");
    }
  };
  handleCommand = function(type) {
    var handled;
    handled = false;
    _.each(commandWraps, function(cmd) {
      if (cmd.type === type && cmd.name === process.argv[0]) {
        handled = true;
        return cmd.callback();
      }
    });
    return handled;
  };
  missingPythonDeps = function(commandName, packageName) {
    log(" * missing " + commandName + " !");
    log("   try... pip install " + packageName);
    log("   or...  easy_install install " + packageName);
    return process.exit(-1);
  };
  exports.run = function() {
    var ok_callback;
    showGreatings();
    ok_callback = function() {
      var options;
      if (!handleCommand('before')) {
        options = process.argv.join(' ');
        log("Calling couchapp");
        return exec("couchapp " + options, handleOutput(function() {
          return handleCommand('after');
        }));
      }
    };
    return exec('couchdb-dump --version', handleOutput(function() {
      return exec('couchapp --version', handleOutput(ok_callback, function() {
        return missingPythonDeps("couchapp", "couchapp");
      }));
    }, function() {
      return missingPythonDeps("couchdb-dump", "couchdb");
    }));
  };
}).call(this);
